# Warewulf shared library
#
# This script is a component of Warewulf,
# http://www.runlevelzero.net/greg/warewulf
#
#########################################################################
#
# Copyright (c) 2003, The Regents of the University of California, through
# Lawrence Berkeley National Laboratory (subject to receipt of any
# required approvals from the U.S. Dept. of Energy).  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# The GNU GPL Document can be found at:
# http://www.gnu.org/copyleft/gpl.html
#
#########################################################################
#
# Written and maintained by:
#       Greg Kurtzer, <gmkurtzer@lbl.gov>

my $debug = ();
#$debug = 1;

package Warewulf::Config;
use File::Basename;
use Warewulf::Util;
use Socket;
use Exporter;
@ISA = ('Exporter');

@EXPORT = qw (
   $node_config_template
   &node_config
   &group_config
   &node_status
   &master_config
   &get_net_conf
   &read_conf
   &ipcalc_network
   &client_config
);


our $node_config_template = "# If no value is present, the entry is ignored.

   desc                 = Default node configuration
   hardware addr        = #hwaddr#
   admin ipaddr         = #adminip#
   sharedfs ipaddr      = #sharedfsip#
   cluster ipaddr       = #clusterip#
   gateway ip           =
   other dev            =
   other addr           =

# The following should already be configured via the node group but you can
# override those values on a per node basis here:

   user names           =
   user groups          =
   vnfs                 =
   admin device         =
   cluster device       =
   sharedfs device      =
   kernel image         =
   wwinitrd image       =
";

sub read_conf {
   my ($file, @NULL) = @_;
   my ( %ret );
   open(CONF, "$file");
   while(<CONF>) {
      chomp;
      $_ =~ s/#.*$//g;
      $_ =~ s/^\s+//g;
      $_ =~ s/\s+$//g;
      $_ =~ s/\s*=\s*/=/g;
      if ( $_ =~ /^([a-zA-Z0-9_\s]+)=(.)(.*)(.)$/) {
         $key = $1;
         if ( $ret{"$key"} ) {
            $ret{"$key"} .= " ";
         }
         if ( ( $2 eq '"' and $4 eq '"') or ( $2 eq '\'' and $4 eq '\'') ){
            $ret{"$key"} .= "$3";
         } elsif ( $2 eq '"' or $2 eq '\'' ) {
            $open_quote = '1';
            $ret{"$key"} .= "$3$4";
         } else {
            $ret{"$key"} .= "$2$3$4";
         }
      } elsif ( $open_quote ) {
         if ( $_ =~ /^(.+)(.)$/ ) {
            if ( $2 eq '"' ) {
               $ret{"$key"} .= " $1 ";
               $open_quote = ();
            } else {
               $ret{"$key"} .= " $1$2 ";
            }
         }
      }
   }
   close CONF;
   return %ret;
}

sub ipcalc_network {
   my ( $ip, $mask, @null) = @_;
   my ( $i );
   my @ip = split /\./, $ip;
   my @mask = split /\./, $mask;
   for ($i=0; $i<4; ++$i) {
      $ip[$i] = ($ip[$i]+0) & ($mask[$i]+0);
   }
   $ip = join('.', @ip);
   return($ip);
}

sub node_config {
   my ( $node, $group, %group_hash, %nodes, %tmp);
   chdir("/etc/warewulf/nodes/");
   @groups = glob("*");
   foreach $group ( @groups ) {
      next if ( $group eq "disabled" );
      next if ( ! -d "/etc/warewulf/nodes/$group" );
      if ( ! -f "/etc/warewulf/nodes/$group/config" and $group ne 'new' ) {
         warn "WARNING: Configuration file not found! '/etc/warewulf/nodes/$group/config'\n";
      }
      chdir("/etc/warewulf/nodes/$group");
      %group_hash = &read_conf("/etc/warewulf/nodes/$group/config");
      @nodes = glob("*");
      foreach $node ( @nodes ) {
         next if ( $node eq "config" );
         %tmp = &read_conf("/etc/warewulf/nodes/$group/$node");
         if ( ! $tmp{"hardware addr"} ) {
            next;
         }
         %{$nodes{"$node"}} = %group_hash;
         $nodes{"$node"}{"group"} = $group;
         foreach $t ( keys %tmp ) {
            $nodes{"$node"}{"$t"} = $tmp{"$t"};
         }
      }
   }
   return(%nodes);
}

sub group_config {
   my ( $node, $group, %group_hash, %nodes, %tmp);
   chdir("/etc/warewulf/nodes/");
   @groups = glob("*");
   foreach $group ( @groups ) {
      next if ( ! -f "$group/config" );
#      chdir("/etc/warewulf/nodes/$group");
      %{$group_hash{$group}} = &read_conf("/etc/warewulf/nodes/$group/config");
   }
   return(%group_hash);
}

sub node_addr {
   my ($node_num, $network, $mask, @null) = @_;
   my $net_bin = unpack("N", inet_aton("$network"));
   my $mask_bin = unpack("N", inet_aton("$mask"));
   my $twiddle = ((~$mask_bin + 1) & 0xffffffff) >> 2;
   my $addr = ( $net_bin & $mask_bin ) | ( $node_num & ~$mask_bin ) | $twiddle;
   $addr = inet_ntoa(pack('N',"$addr"));
   return $addr;
}

sub node_status {
   my ( $host, $port, $timeout, @NULL ) = @_;
   my ( %nodes, %nodecfg, @s, $entry, $value, $nodename, $sock );

   unless ( $host ) {
      $host = 'localhost';
   }
   unless ( $port ) {
      $port = '9873';
   }
   unless ( $timeout ) {
      $timeout = '5';
   }

   my $sock = new IO::Socket::INET ( PeerAddr => $host,
                                     PeerPort => $port,
                                     Proto    => 'tcp',
                                     Timeout  => $timeout,
                                   );
   unless ( $sock ) {
      warn "Could not connect to $host:$port!\n";
   }

   foreach ( keys %nodecfg ) {
      $nodes{$_}{NODESTATUS} = 'DOWN';
      $nodes{$_}{LASTCONTACT} = 'never';
   }

   while (<$sock>) {
      chomp;
      ( $entry, $value ) = split (/=/, $_ );
      next unless ( defined $entry and defined $value );
      if ( $entry eq 'NODE' ) {
         $nodename = $value;
         next;
      }
      if ( $nodename ) {
         $nodes{$nodename}{$entry} = $value;
      }
   }
   return(%nodes);
}

sub get_net_conf {
   my ( $device, $ifcfg_path ) = @_;
   my ( $ipaddr, $network, $netmask, $entry, $value);
   if ( ( $device =~ /^.+:\d+$/ and -f "${ifcfg_path}$device" ) or -f "${ifcfg_path}$device" ) {
      open(IFCFG,"${ifcfg_path}$device")
            or die "Could not open: ${ifcfg_path}$device\n   as defined in /etc/warewulf/master.conf!\n";
      while(<IFCFG>) {
         chomp;
         next unless $_;
         $_ =~ s/#.*$//g;
         $_ =~ s/^\s+//g;
         $_ =~ s/\s+$//g;
         $_ =~ s/\s*=\s*/=/g;
         ($entry, $value) = split (/=/, $_);
         if ( $entry eq 'IPADDR' ) {
            $ipaddr = $value;
         } elsif ( $entry eq 'NETWORK' ) {
            $network = $value;
         } elsif ( $entry eq 'NETMASK' ) {
            $netmask = $value;
         }
      }
      if ( ! $ipaddr ) {
         warn "$device has no configuration for IPADDR!\n";
      }
      if ( ! $netmask ) {
         warn "$device has no configuration for NETMASK!\n";
      }
      if ( ! $network ) {
         unless ( $network = &ipcalc_network($ipaddr, $netmask) ) {
            warn "$device could not obtain the NETWORK!\n";
         }
      }
   }
   return($ipaddr, $network, $netmask);
}

sub master_config {
   my ( @config, %master, $current, $entry, $value );
   open(MASTERCFG, "< /etc/warewulf/master.conf");
   while(<MASTERCFG>) {
      chomp;
      $_ =~ s/#.*$//g;
      $_ =~ s/^\s+//g;
      $_ =~ s/\s+$//g;
      next unless $_;
      if ( $_ =~ /^\s*\[([\w+-\.]+)\]\s*$/ ) {
         $current = $1;
      } else {
         ($entry, $value) = split(/\s+=\s+/, $_);
         $value =~ s/\s+$//g;
         $value =~ s/`(.+)`/`echo -n \`$1\``/eg;
         $master{$current}{"$entry"} = $value;
      }
   }

   if ( $master{'network'}{'admin device'} ) {
      ( $ipaddr, $network, $netmask ) = &get_net_conf(
            $master{'network'}{'admin device'}, 
            $master{'paths'}{'ifcfg'});
      $master{'network'}{'admin ipaddr'} = $ipaddr;
      $master{'network'}{'admin network'} = $network;
      $master{'network'}{'admin netmask'} = $netmask;
   } else {
      warn "You need to configure the master node's admin device!\n";
      warn "\t(/etc/warewulf/master.conf:admin device = ???)\n";
      exit 1;
   }
   if ( $master{'network'}{'cluster device'} ) {
      ( $ipaddr, $network, $netmask ) = &get_net_conf(
            $master{'network'}{'cluster device'}, 
            $master{'paths'}{'ifcfg'});
      $master{'network'}{'cluster ipaddr'} = $ipaddr;
      $master{'network'}{'cluster network'} = $network;
      $master{'network'}{'cluster netmask'} = $netmask;
   } else {
      warn "You need to configure the master node's cluster device!\n";
      warn "\t(/etc/warewulf/master.conf:cluster device = ???)\n";
      exit 1;
   }
   if ( $master{'network'}{'sharedfs device'} ) {
      ( $ipaddr, $network, $netmask ) = &get_net_conf(
            $master{'network'}{'sharedfs device'}, 
            $master{'paths'}{'ifcfg'});
      $master{'network'}{'sharedfs ipaddr'} = $ipaddr;
      $master{'network'}{'sharedfs network'} = $network;
      $master{'network'}{'sharedfs netmask'} = $netmask;
   } else {
      warn "You need to configure the master node's sharedfs device!\n";
      warn "\t(/etc/warewulf/master.conf:sharedfs device = ???)\n";
      exit 1;
   }

   if ( $master{'network'}{'boot device'} and $master{'network'}{'boot device'} ne "auto" ) {
      ( $ipaddr, $network, $netmask ) = &get_net_conf(
            $master{'network'}{'boot device'},
            $master{'paths'}{'ifcfg'});
   } else {
      ( $ipaddr, $network, $netmask ) = &get_net_conf(
            "$master{'network'}{'admin device'}:1",
            $master{'paths'}{'ifcfg'});
   }
   $master{'ipaddrs'}{'boot'} = $ipaddr;
   $master{'network'}{'boot network'} = $network;
   $master{'network'}{'boot netmask'} = $netmask;

   $master{'conjoined net'}{'checksum'} = 0;
   $master{'conjoined net'}{'cluster'} = 0;
   $master{'conjoined net'}{'sharedfs'} = 0;
   if ($master{'network'}{'cluster device'} eq $master{'network'}{'admin device'}) {
      $master{'conjoined net'}{'checksum'} += 1;
      $master{'conjoined net'}{'cluster'} += 1;
   }
   if ($master{'network'}{'sharedfs device'} eq $master{'network'}{'admin device'}) {
      $master{'conjoined net'}{'checksum'} += 2;
      $master{'conjoined net'}{'sharedfs'} += 2;
   }
   if ($master{'network'}{'sharedfs device'} eq $master{'network'}{'cluster device'}) {
      $master{'conjoined net'}{'checksum'} += 4;
      $master{'conjoined net'}{'sharedfs'} += 4;
   }
   return(%master);
}

sub client_config {
   my %return = &read_conf("/etc/warewulf/client.conf");
   return(%return);
}



1;
