#!/bin/sh

MIRROR=http://altruistic.lbl.gov/mirrors/caosity/cAos-2

if [ "x$1" != "x" ]; then
   NAME=$1
else
   NAME=caos2
fi

echo "Building in: /vnfs/$NAME/"


YUM_PACKAGE_COMMAND=" install
   SysVinit basesystem bash caos-release chkconfig coreutils cracklib
   cracklib-dicts db4 dev-minimal e2fsprogs ethtool filesystem findutils gawk
   glib glibc glibc-common grep info initscripts iproute iputils krb5-libs
   libacl libattr libgcc libselinux libsepol libstdc++ libtermcap mingetty
   mktemp module-init-tools ncurses net-tools nfs-utils nscd pam pcre popt
   portmap procps psmisc rdate rsh rsh-server rsync sed setup shadow-utils
   sysklogd tcp_wrappers termcap tzdata udev util-linux warewulf-wulfd words
   xinetd zlib hwdata usbutils MAKEDEV hotplug
"

mkdir -p /vnfs/$NAME/dev/
mkdir -p /vnfs/$NAME/etc/sysconfig/
mkdir -p /vnfs/$NAME/etc/warewulf/
mkdir -p /vnfs/$NAME/var/lib/rpm
rpm --initdb --dbpath /vnfs/$NAME/var/lib/rpm
cat <<EOF >/vnfs/$NAME/etc/yum.conf
[main]
cachedir=/var/cache/yum
debuglevel=2
logfile=/var/log/yum.log
pkgpolicy=newest
distroverpkg=caos-release
installonlypkgs=linux linux-smp kernel kernel-smp
exactarch=1

[core]
name=cAos2 core system repository
baseurl=$MIRROR/core/\$basearch

[ext-current]
name=cAos2 extended current system repository
baseurl=$MIRROR/ext/current/\$basearch
EOF

cat <<EOF > /vnfs/$NAME/etc/warewulf/node-partitions
# USAGE: cat /etc/warewulf/node-partitions | sfdisk [device]
#
# This will create 2 partitions on device. First is a swap of about 2Gb,
# and the second is the remainder of the filesystem

,254,82
,,83
EOF


mknod /vnfs/$NAME/dev/null c 1 3		>/dev/null 2>&1
yum -ty -d 2 --installroot /vnfs/$NAME $YUM_PACKAGE_COMMAND

rpm --root /vnfs/$NAME -e --nodeps linux	>/dev/null 2>&1
rpm --root /vnfs/$NAME -e --nodeps linux-smp	>/dev/null 2>&1
rpm --root /vnfs/$NAME -e --nodeps dev		>/dev/null 2>&1

echo "NETWORKING=yes" 	> /vnfs/$NAME/etc/sysconfig/network
echo "rsh" 		>> /vnfs/$NAME/etc/securetty
echo "rlogin" 		>> /vnfs/$NAME/etc/securetty
echo "rexec" 		>> /vnfs/$NAME/etc/securetty
echo "127.0.0.1		localhost localhost.localdomain" \
			> /vnfs/$NAME/etc/hosts

if [ -x "/vnfs/$NAME/usr/sbin/pwconv" ]; then
   /usr/sbin/chroot /vnfs/$NAME /usr/sbin/pwconv
fi
if [ -x "/vnfs/$NAME/sbin/chkconfig" ]; then
   /usr/sbin/chroot /vnfs/$NAME /sbin/chkconfig rsh on
   /usr/sbin/chroot /vnfs/$NAME /sbin/chkconfig rlogin on
   /usr/sbin/chroot /vnfs/$NAME /sbin/chkconfig rexec on
   /usr/sbin/chroot /vnfs/$NAME /sbin/chkconfig xinetd on
   /usr/sbin/chroot /vnfs/$NAME /sbin/chkconfig wulfd on
fi

cat <<EOF >/vnfs/$NAME/etc/fstab
# Don't touch the following macro unless you really know what your doing!
%{root entry}
none            /dev/pts                devpts  gid=5,mode=620  0 0
none            /dev/shm                tmpfs   defaults        0 0
none            /proc                   proc    defaults        0 0

# To use a local disk on the nodes (make sure the mountpoints exist in
# the VNFS!!!)
#/dev/hda2      /scratch                ext3    defaults        0 0
#/dev/hda1      none                    swap    defaults        0 0

# The following lines are macros for the master's shared NFS volumes. You can
# comment out or remove if you have other plans for sharing the users data.
%{sharedfs entry}
%{vnfs entry}
EOF


/usr/sbin/chroot /vnfs/$NAME umount /proc	>/dev/null 2>&1
/usr/sbin/chroot /vnfs/$NAME umount /sys	>/dev/null 2>&1

ln -s ../bin/true /vnfs/$NAME/sbin/detect	>/dev/null 2>&1
ln -s ../bin/true /vnfs/$NAME/sbin/nash		>/dev/null 2>&1
rm -rf /vnfs/$NAME/var/cache/yum/*
rm -rf /vnfs/$NAME/var/log/*

# add system root password to the nodes
umask 277		# to prevent user readable files
echo "configuring root account in VNFS image"
sed -e s/root::/root:!!:/ < /vnfs/$NAME/etc/shadow > /vnfs/$NAME/etc/shadow.new
cp /vnfs/$NAME/etc/shadow.new /vnfs/$NAME/etc/shadow
rm /vnfs/$NAME/etc/shadow.new
umask 0022		# set umask bacl to default
echo ""
echo "The root account in the VNFS is currently locked."


if [ ! -e "/vnfs/default" ]; then
   ln -s $NAME /vnfs/default
fi



