#!/bin/sh

#
# centos4-vnfs.sh version 1.2
#########################################################################
#
# Copyright (c) 2003, The Regents of the University of California, through
# Lawrence Berkeley National Laboratory (subject to receipt of any
# required approvals from the U.S. Dept. of Energy).  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# The GNU GPL Document can be found at:
# http://www.gnu.org/copyleft/gpl.html
#
#########################################################################
#
# galitz at berkeley.edu
# geoff at galitz.org
#
#	current as of November 2005
#	galitz

# use the LBL mirror
MIRROR=http://altruistic.lbl.gov/mirrors/centos/
releasever=4
export releasever



#check for SELinux as the default policy interferes with 
#rpm scriptlets run by warewulf
[ -f /usr/sbin/selinuxenabled ] && {
	/usr/sbin/selinuxenabled
	[ $? = 0 ] && {
		echo -e "\n"
		echo "SELinux is enabled.  "
		echo "SELinux interferes with configuration of the VNFS.  Please"
		echo "disable SELinux in /etc/sysconfig/selinux, reboot and then"
		echo "run this script."
		echo -e "\n"	
		echo "Exiting..."
		exit 1
	}
}


if [ "x$1" != "x" ]; then
   NAME=$1
else
   NAME=centos-4
fi

echo "Building in: /vnfs/$NAME/"


mkdir -p /vnfs/$NAME/etc/sysconfig
mkdir -p /vnfs/$NAME/var/lib/rpm
mkdir -p /vnfs/$NAME/var/lock/rpm
rpm --initdb --dbpath /vnfs/$NAME/var/lib/rpm
cat <<EOF >/vnfs/$NAME/etc/yum.conf
[main]
cachedir=/var/cache/yum
debuglevel=2
logfile=/var/log/yum.log
pkgpolicy=newest
distroverpkg=centos-release
installonlypkgs=linux linux-smp kernel kernel-smp
exactarch=1
reposdir=/dev/null

#reposdir=/dev/null is needed to prevent yum from parsing
#through local repos on the server and possibly misinterpreting
#the release and arch of the VNFS.

[base]
name=CentOS-\$releasever - Base
baseurl=$MIRROR/4/os/\$basearch

[updates]
name=CentOS-$releasever - Updates
baseurl=$MIRROR/4/updates/\$basearch

[dag] 
name=Dag RPM Repository for Red Hat Enterprise Linux 
baseurl=http://apt.sw.be/redhat/el4/en/\$basearch/dag 
gpgcheck=0
enabled=1 

[kbs-CentOS-Extras]
name=CentOS.Karan.Org-EL\$releasever - Stable
gpgcheck=0
enabled=1
baseurl=http://centos.karan.org/el4/extras/stable/\$basearch/RPMS/

# NOTE:  kbs repo is needed for the Unix::Syslog perl module


EOF

mkdir -p /vnfs/$NAME/dev/
mknod /vnfs/$NAME/dev/null c 1 3		>/dev/null 2>&1
yum -y -t -d 2 --disablerepo=updates -c /vnfs/$NAME/etc/yum.conf --installroot /vnfs/$NAME install \
   SysVinit basesystem bash centos-release chkconfig coreutils cracklib \
   cracklib-dicts db4 e2fsprogs ethtool filesystem findutils gawk \
   glib glibc glibc-common grep initscripts iproute iputils krb5-libs \
   libacl libattr libgcc libstdc++ libtermcap mingetty mktemp ncurses \
   net-tools nfs-utils pam pcre popt portmap procps psmisc rdate rsh \
   rsh-server rsync sed setup shadow-utils sysklogd tcp_wrappers termcap \
   tzdata util-linux warewulf-wulfd words xinetd zlib tar mkinitrd less \
   gzip which util-linux module-init-tools udev

rpm --root /vnfs/$NAME -e --nodeps kernel	>/dev/null 2>&1
rpm --root /vnfs/$NAME -e --nodeps kernel-smp	>/dev/null 2>&1
rpm --root /vnfs/$NAME -e --nodeps dev		>/dev/null 2>&1

echo "NETWORKING=yes"	> /vnfs/$NAME/etc/sysconfig/network
echo "rsh"		>> /vnfs/$NAME/etc/securetty
echo "rlogin"		>> /vnfs/$NAME/etc/securetty
echo "rexec"		>> /vnfs/$NAME/etc/securetty
echo "127.0.0.1		localhost localhost.localdomain" \
			> /vnfs/$NAME/etc/hosts

if [ -x "/vnfs/$NAME/usr/sbin/pwconv" ]; then
   /usr/sbin/chroot /vnfs/$NAME /usr/sbin/pwconv
   >/dev/null 2>&1||:
fi
if [ -x "/vnfs/$NAME/sbin/chkconfig" ]; then
   /usr/sbin/chroot /vnfs/$NAME /sbin/chkconfig rsh on >/dev/null 2>&1
   /usr/sbin/chroot /vnfs/$NAME /sbin/chkconfig rlogin on >/dev/null 2>&1
   /usr/sbin/chroot /vnfs/$NAME /sbin/chkconfig rexec on >/dev/null 2>&1
   /usr/sbin/chroot /vnfs/$NAME /sbin/chkconfig xinetd on >/dev/null 2>&1
fi

cat <<EOF >/vnfs/$NAME/etc/fstab
# Don't touch the following macro unless you really know what your doing!
%{root entry}
none            /dev/pts                devpts  gid=5,mode=620  0 0
none            /dev/shm                tmpfs   defaults        0 0
none            /proc                   proc    defaults        0 0

# To use a local disk on the nodes (make sure the mountpoints exist in
# the VNFS!!!)
#/dev/hda2      /scratch                ext3    defaults        0 0
#/dev/hda1      none                    swap    defaults        0 0

# The following lines are macros for the master's shared NFS volumes. You can
# comment out or remove if you have other plans for sharing the users data.
%{sharedfs entry}
%{vnfs entry}
EOF

touch /vnfs/$NAME/fastboot


/usr/sbin/chroot /vnfs/$NAME umount /proc	>/dev/null 2>&1
rm -rf /vnfs/$NAME/var/cache/yum/*
rm -rf /vnfs/$NAME/var/log/*

# add broken_shadow to pam.d/system-auth to allow users 
# to not have shadow entries fix courtesy of padowns
mv /vnfs/$NAME/etc/pam.d/system-auth /vnfs/$NAME/etc/pam.d/system-auth-orig
sed -e '/^account.*pam_unix\.so$/s/$/\ broken_shadow/' \
/vnfs/$NAME/etc/pam.d/system-auth-orig > /vnfs/$NAME/etc/pam.d/system-auth
rm -f /vnfs/$NAME/etc/pam.d/system-auth-orig

# add system root password to the nodes
umask 277               # to prevent user readable files
echo "configuring root account in VNFS image"
sed -e s/root::/root:!!:/ < /vnfs/$NAME/etc/shadow > /vnfs/$NAME/etc/shadow.new
cp /vnfs/$NAME/etc/shadow.new /vnfs/$NAME/etc/shadow
rm /vnfs/$NAME/etc/shadow.new
umask 0022              # set umask back to default



if [ ! -e "/vnfs/default" ]; then
   ln -s $NAME /vnfs/default
fi

echo "Finished."
echo "Make sure you install warewulf-wuld to the node image."
echo "The root account in the VNFS image is locked."

