#!/bin/sh
#
# This script is contributed by:
#   slaton <slaton@OCF.Berkeley.EDU>

## use Stanford's tao mirror; other mirrors listed here:
## http://taolinux.org/?q=node/view/17#mirrors
##
MIRROR=http://mirror.stanford.edu/yum/pub/taolinux

if [ "x$1" != "x" ]; then
   NAME=$1
else
   NAME=tao-1.0
fi

echo "Building in: /vnfs/$NAME/"


mkdir -p /vnfs/$NAME/etc/sysconfig/
mkdir -p /vnfs/$NAME/var/lib/rpm
rpm --initdb --dbpath /vnfs/$NAME/var/lib/rpm
cat <<EOF >/vnfs/$NAME/etc/yum.conf
[main]
cachedir=/var/cache/yum
debuglevel=2
logfile=/var/log/yum.log
pkgpolicy=newest
distroverpkg=taolinux
installonlypkgs=linux linux-smp kernel kernel-smp
exactarch=1

[base]
name=Tao Linux 1.0 - Base
baseurl=$MIRROR/tao-1.0-\$basearch/dist/

[updates]
name=Tao Linux 1.0 - Updates
baseurl=$MIRROR/tao-1.0-\$basearch/updates/
EOF

mkdir -p /vnfs/$NAME/dev/
mknod /vnfs/$NAME/dev/null c 1 3		>/dev/null 2>&1

## 050414 - slaton@ocf.berkeley.edu
##          modified from centos33 script.
##
##   added:	tao-release (required)
##		tcsh
##
##   removed:	centos-release
##		dev-minimal (not available from tao)
##		warewulf-wulfd (not available from tao)

## IMPORTANT: dev-minimal and warewulf-wulfd src.rpms must be built & installed
## separately. dev-minimal can be obtained from:
## http://warewulf.lbl.gov/downloads/dependancies/MAKEDEV-3.3.8-5.ww.src.rpm


yum -t -d 2 --installroot /vnfs/$NAME install \
   SysVinit basesystem bash chkconfig coreutils cracklib cracklib-dicts db4 \
   e2fsprogs ethtool filesystem findutils gawk glib glibc glibc-common grep \
   info initscripts iproute iputils krb5-libs libacl libattr libgcc \
   libstdc++ libtermcap mingetty mktemp ncurses net-tools nfs-utils nscd pam \
   pcre popt portmap procps psmisc rdate rsh rsh-server rsync sed setup \
   shadow-utils sysklogd tao-release tcp_wrappers tcsh termcap tzdata \
   util-linux words xinetd zlib tar gpm mkinitrd less mount modutils lvm gzip \
   which losetup

rpm --root /vnfs/$NAME -e --nodeps kernel	>/dev/null 2>&1
rpm --root /vnfs/$NAME -e --nodeps kernel-smp	>/dev/null 2>&1
rpm --root /vnfs/$NAME -e --nodeps dev		>/dev/null 2>&1

echo "NETWORKING=yes" 	> /vnfs/$NAME/etc/sysconfig/network
echo "rsh" 		>> /vnfs/$NAME/etc/securetty
echo "rlogin" 		>> /vnfs/$NAME/etc/securetty
echo "rexec" 		>> /vnfs/$NAME/etc/securetty
echo "127.0.0.1		localhost localhost.localdomain" \
			> /vnfs/$NAME/etc/hosts

if [ -x "/vnfs/$NAME/usr/sbin/pwconv" ]; then
   /usr/sbin/chroot /vnfs/$NAME /usr/sbin/pwconv
   >/dev/null 2>&1||:
fi
if [ -x "/vnfs/$NAME/sbin/chkconfig" ]; then
   /usr/sbin/chroot /vnfs/$NAME /sbin/chkconfig rsh on >/dev/null 2>&1
   /usr/sbin/chroot /vnfs/$NAME /sbin/chkconfig rlogin on >/dev/null 2>&1
   /usr/sbin/chroot /vnfs/$NAME /sbin/chkconfig rexec on >/dev/null 2>&1
   /usr/sbin/chroot /vnfs/$NAME /sbin/chkconfig xinetd on >/dev/null 2>&1
fi

cat <<EOF >/vnfs/$NAME/etc/fstab
# Don't touch the following macro unless you really know what your doing!
%{root entry}
none            /dev/pts                devpts  gid=5,mode=620  0 0
none            /dev/shm                tmpfs   defaults        0 0
none            /proc                   proc    defaults        0 0

# To use a local disk on the nodes (make sure the mountpoints exist in
# the VNFS!!!)
#/dev/hda2      /scratch                ext3    defaults        0 0
#/dev/hda1      none                    swap    defaults        0 0

# The following lines are macros for the master's shared NFS volumes. You can
# comment out or remove if you have other plans for sharing the users data.
%{sharedfs entry}
%{vnfs entry}
EOF

touch /vnfs/$NAME/fastboot


/usr/sbin/chroot /vnfs/$NAME umount /proc	>/dev/null 2>&1
rm -rf /vnfs/$NAME/var/cache/yum/*
rm -rf /vnfs/$NAME/var/log/*


if [ ! -e "/vnfs/default" ]; then
   ln -s $NAME /vnfs/default
fi
