#!/usr/bin/perl


#
#  vnfs-remote.pl version 2.0
#########################################################################
#
# Copyright (c) 2003, The Regents of the University of California, through
# Lawrence Berkeley National Laboratory (subject to receipt of any
# required approvals from the U.S. Dept. of Energy).  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# The GNU GPL Document can be found at:
# http://www.gnu.org/copyleft/gpl.html
#
#########################################################################
#
#  Tue May 24 12:17:10 PDT 2005
#  galitz at berkeley.edu
#  geoff at galitz.org
#
#  This script will create a VNFS out of an installed filesystem on 
#  a system via rsync.  The system can be of any type.  The image name
#  and the rsync module are assumed to be the same.
#
#  Note:  The VNFS will not be compressed and can get quite large
#	  The /proc and /sys filesystems are not transmitted (the
#	  mountpoints are).
#
#  To do items:  
#	- Add support to override rsync-module
#	- Add support for multple rsync daemons (in case the master is 
#	  already running one).
#
#

use Getopt::Long;


GetOptions(
	'help'        => \$help,
	'rsh'		=> \$rsh,
	'ssh'		=> \$ssh
);


$image = @ARGV[1];
$remote_host = @ARGV[0];
$local_image = "/vnfs/$image";



# for pulling, we will call rsync and make a connection to 
# to the remote system.


if ( $help ) {
	print "Usage: vnfs-remote.pl [options] image remote-host \n\n";
	print "Options:\n";
	print "\t --help   prints a short help summary\n";
	print "\t --rsh    use rsh for vnfs copying.\n\n";
	print "vnfs-remote will create a VNFS out of an installed filesystem for you via rsync.\n";
	print "\nFor example,\n";
	print "vnfs-remote custom-image image.machine.com \n";
	print "\n\n";
	exit;
}

print "vnfs-remote --help for usage information.\n";

if ( ! "$image" ) {
	$image = "default";
}
if ( ! -d "/vnfs/$image" ) {
	print "Creating /vnfs/$image\n";
	system ("mkdir /vnfs/$image");
}
# do we need to create a default VNFS?
if (  ! -d "/vnfs/default" ) {
	system ("ln -s $local_image /vnfs/default");
} 
if ( ! "$remote_host" ) {
	print "I need a remote host to pull from.\n";
	print "vnfs-remote.pl --help for more info.\n";
	exit 1;
}

if ( $rsh ) {
	$conn = "rsh";
} else {
	$conn = "ssh";
}
if ( $ssh and $rsh) {
	print "Cannot use both ssh and rsh at the same time.";
	exit 2;
}
	print "Ok... pull from $remote_host via $conn \n";
system ("cd $local_image ; rsync -avz -e $conn $remote_host:/  --exclude=/proc/* --exclude=/sys/* --exclude=/home/*  --exclude=/media/* --exclude=/mnt/* --exclude=/boot/* --exclude=/opt/* --exclude=/tmp/* --exclude=/usr/src/* --exclude=/usr/local/src/* $local_image");
print "Done!\n";
exit;
